CON

  _clkmode      = xtal1 + pll16x
  _xinfreq      = 5_000_000

  AUDIO         = 26                            'P26 is audio output for Prop BOE.  

OBJ

  fir   :       "rain_filter_1"

PUB  start

  cognew(@rain, fir.start(@busy)) 

DAT

              org       0
rain          mov       arg_addr,par            'Get busy address.
              add       arg_addr,#4             'Next long up is argument address.
              mov       drop_addr,par           'Get address of droplet waveform table.
              add       drop_addr,#8
              mov       dira,dira0              'Initialize dira for audio output.
              mov       ctra,ctra0              'Set up counter for DUTY mode.
              mov       period,cnt              'Establish first time sync.
              add       period,_22khz
        
main_lp       call      #random                 'Get next random number for white noise.
              mov       arg,rand                'Save in arg.
              wrlong    arg,arg_addr            'Write arg to hub.
              wrlong    one,par                 'Tell FIR filter we're ready with next sample.
              
              cmps      drop_index,max_index wc 'Has last droplet sound finished? 
        if_c  jmp       #:do_drop               '  No:  Go work on it som more.

              call      #random                 '  Yes: Pick a number, any number.
              cmp       rand,rand_min wc        '       Start a new droplet now?
        if_nc jmp       #:wait_lp               '         No: Just the noise.

              neg       drop_index,#103 * 4     'Initialize drop_index to -4 * actual starting index.
              call      #random                 'Set a random droplet waveform frequency
              mov       drop_freq,rand          '  between 10 and 41 (2.5 and 10.25 actual).          
              shr       drop_freq,#27
              add       drop_freq,#10
              mov       max_index,drop_freq     'Make total actual drop time constant.
              shl       max_index,#10

:do_drop      mov       drop,#0                 'Initialize drop, in case in silent trailer.
              abs       drop_ptr,drop_index     'Point to positive part of table.
              shr       drop_ptr,#2             'Divide index by 4.
              cmps      drop_ptr,#255 wc        'Is it past end of table>
        if_nc jmp       #:add_index             '  Yes: Leave drop at zero.
        
              add       drop_ptr,drop_addr      '  No:  Add table address to pointer.
              rdlong    drop,drop_ptr           '       Get the waveform value there.
              shl       drop,#24                '       Convert to twos-complement.
              sar       drop,#3                 '       Attenuate.
:add_index    add       drop_index,drop_freq    'Increment table index by frequency.

:wait_lp      rdlong    arg,par wz              'Wait for FIR filter to do its thing.
        if_nz jmp       #:wait_lp

              rdlong    arg,arg_addr            'Get filtered result.
              sar       arg,#2
              add       arg,drop
              
skip_filter   add       arg,zero                'Add the AC zero level.
              waitcnt   period,_22khz           'Wait for next time sync.
              mov       frqa,arg                'Write output to counter.
              jmp       #main_lp                'Back for another.

'-------[ Random: generate 32-bit random number in rand ]----------------------

random        mov       ctr,#32
                               
:randlp       test      rand,#%10111 wc            
              rcr       rand,#1                    
              djnz      ctr,#:randlp

random_ret    ret

'-------[ Constants & initialized variables ]----------------------------------

dira0         long      1 << AUDIO
ctra0         long      %00110 << 26 | AUDIO 
rand          long      1
_22khz        long      80_000_000 / 22_000
one           long      1
zero          long      $8000_0000
drop_index    long      40000      
max_index     long      40000
drop          long      0
rand_min      long      $20_0000                'Make larger for more frequent drops.

'-------[ Uninitialized variables ]--------------------------------------------

arg_addr      res       1
drop_addr     res       1
arg           res       1
period        res       1
ctr           res       1
drop_incr     res       1
drop_ptr      res       1
drop_freq     res       1

'-------[ Droplet waveform ]---------------------------------------------------

busy          long      0      
data          long      0
droplet       byte      $7f,$7e,$7e,$7d,$7c,$7b,$7a,$78,$76,$74,$71,$6e,$6b,$68,$65,$61
              byte      $5d,$59,$55,$51,$4c,$47,$42,$3d,$38,$33,$2d,$28,$22,$1d,$17,$11
              byte      $0c,$06,$00,$fc,$f6,$f0,$eb,$e5,$df,$da,$d5,$d0,$ca,$c6,$c1,$bc
              byte      $b8,$b3,$af,$ab,$a7,$a4,$a0,$9d,$9a,$98,$95,$93,$91,$8f,$8e,$8d
              byte      $8c,$8b,$8b,$8a,$8a,$8b,$8b,$8c,$8d,$8e,$90,$92,$94,$96,$98,$9b
              byte      $9e,$a1,$a4,$a8,$ab,$af,$b3,$b7,$bb,$c0,$c4,$c9,$cd,$d2,$d7,$dc
              byte      $e1,$e6,$eb,$f0,$f5,$fa,$ff,$02,$07,$0c,$11,$16,$1a,$1f,$24,$28
              byte      $2c,$30,$34,$38,$3c,$3f,$43,$46,$49,$4c,$4e,$51,$53,$55,$57,$59
              byte      $5a,$5c,$5d,$5d,$5e,$5f,$5f,$5f,$5f,$5e,$5e,$5d,$5c,$5b,$59,$58
              byte      $56,$54,$52,$50,$4d,$4b,$48,$45,$43,$3f,$3c,$39,$36,$32,$2f,$2b
              byte      $28,$24,$20,$1c,$19,$15,$11,$0d,$0a,$06,$02,$ff,$fc,$f8,$f4,$f1
              byte      $ed,$ea,$e7,$e4,$e1,$de,$db,$d8,$d5,$d3,$d1,$ce,$cc,$ca,$c8,$c7
              byte      $c5,$c4,$c2,$c1,$c0,$c0,$bf,$bf,$be,$be,$be,$be,$be,$bf,$bf,$c0
              byte      $c1,$c2,$c3,$c4,$c5,$c6,$c8,$c9,$cb,$cd,$cf,$d1,$d3,$d5,$d7,$d9
              byte      $dc,$de,$e0,$e3,$e5,$e8,$ea,$ed,$ef,$f2,$f4,$f7,$f9,$fc,$fe,$00
              byte      $02,$04,$07,$09,$0b,$0d,$0f,$11,$13,$15,$17,$19,$1a,$1c,$1d,$1e
              byte      $20,$21,$22,$23,$24,$25,$25,$26,$27,$27,$27,$28,$28,$28,$28,$28
              byte      $27,$27,$27,$26,$26,$25,$24,$24,$23,$22,$21,$20,$1f,$1e,$1d,$1c
              byte      $1a,$19,$18,$16,$15,$14,$12,$11,$0f,$0e,$0d,$0b,$0a,$08,$07,$05
              byte      $04,$03,$01,$00,$ff,$fe,$fd,$fc,$fa,$f9,$f8,$f7,$f6,$f5,$f4,$f3
              byte      $f2,$f1,$f1,$f0,$ef,$ef,$ee,$ee,$ed,$ed,$ec,$ec,$ec,$ec,$ec,$eb
              byte      $eb,$eb,$eb,$ec,$ec,$ec,$ec,$ec,$ed,$ed,$ed,$ee,$ee,$ef,$ef,$f0
              byte      $f0,$f1,$f2,$f2,$f3,$f3,$f4,$f5,$f6,$f6,$f7,$f8,$f8,$f9,$fa,$fb
              byte      $fb,$fc,$fd,$fd,$fe,$ff,$00,$00,$00,$00,$01,$02,$02,$03,$03,$04
              byte      $04,$05,$05,$06,$06,$06,$07,$07,$07,$08,$08,$08,$08,$09,$09,$09
              byte      $09,$09,$09,$09,$09,$09,$09,$09,$09,$09,$09,$09,$08,$08,$08,$08
              byte      $08,$07,$07,$07,$07,$06,$06,$06,$06,$05,$05,$05,$04,$04,$04,$03
              byte      $03,$03,$02,$02,$02,$01,$01,$01,$01,$00,$00,$00,$00,$00,$00,$00
              byte      $ff,$ff,$ff,$ff,$ff,$fe,$fe,$fe,$fe,$fe,$fe,$fd,$fd,$fd,$fd,$fd
              byte      $fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd,$fd
              byte      $fd,$fd,$fd,$fd,$fd,$fd,$fe,$fe,$fe,$fe,$fe,$fe,$fe,$fe,$fe,$ff
              byte      $ff,$ff,$ff,$ff,$ff,$ff,$ff,$00,$00,$00,$00,$00,$00,$00,$00,$00
